// wserial.cpp

#include <windows.h>
#include <process.h>
#include <stdlib.h>
#include <stdio.h>
#include <mem.h>
#include <timer.h>
#include <serial.h>

typedef struct
{
 unsigned char 	speed;
 unsigned char 	bit;
 unsigned char 	stopbit;
 unsigned char 	parity;
 unsigned short 	rw_size;
 unsigned short 	portid;

 HANDLE           handle;
 COMMTIMEOUTS     CommTimeOuts;
 DCB              dcb;
 COMSTAT          ComState;
 OVERLAPPED       Overlap;
}wserial_TCfg;

static wserial_TCfg	wsi[4];


void serial_Destroy(unsigned short portid)
{
 if (wsi[portid].handle)
 {
  CloseHandle(wsi[portid].handle);
  wsi[portid].handle=NULL;
 }
}

void fillDcb(wserial_TCfg  &wsi,DCB &dcb)
{
 switch (wsi.speed)
 {
  case SERIAL_BAUD1200:{dcb.BaudRate=1200;break;}
  case SERIAL_BAUD2400:{dcb.BaudRate=2400;break;}
  case SERIAL_BAUD4800:{dcb.BaudRate=4800;break;}
  case SERIAL_BAUD9600:{dcb.BaudRate=9600;break;}
  case SERIAL_BAUD19200:{dcb.BaudRate=19200;break;}
 }

 dcb.fBinary            = 1;
 dcb.fOutxCtsFlow       = FALSE;
 dcb.fOutxDsrFlow       = FALSE;
 dcb.fDtrControl        = DTR_CONTROL_HANDSHAKE;
 dcb.fDsrSensitivity    = TRUE;
 dcb.fNull              = FALSE;
 dcb.fRtsControl        = RTS_CONTROL_DISABLE;
 dcb.fAbortOnError      = FALSE;

 switch (wsi.bit)
 {
  case SERIAL_BIT5:{dcb.ByteSize=5;break;}
  case SERIAL_BIT6:{dcb.ByteSize=6;break;}
  case SERIAL_BIT7:{dcb.ByteSize=7;break;}
  case SERIAL_BIT8:{dcb.ByteSize=8;break;}
 }
 switch (wsi.parity)
 {
  case SERIAL_PARITYNOT:{dcb.Parity=NOPARITY;break;}
  case SERIAL_PARITYODD:{dcb.Parity=ODDPARITY;break;}
  case SERIAL_PARITYEVEN:{dcb.Parity=EVENPARITY;break;}
 }

 switch (wsi.stopbit)
 {
   case SERIAL_STOP1:{dcb.StopBits=ONESTOPBIT;break;}
   case SERIAL_STOP2:{dcb.StopBits=TWOSTOPBITS;break;}
 }
}

unsigned short serial_Init(unsigned short portid,
									unsigned char 	speed,
									unsigned char 	bit,
									unsigned char 	stopbit,
									unsigned char 	parity,
									unsigned short rw_size)

{
 char prt[6]="COM ";

 wserial_TCfg  wsi;

 wsi.portid=portid;
 wsi.speed=speed;
 wsi.parity=parity;
 wsi.bit=bit;
 wsi.stopbit=stopbit;
 wsi.rw_size=rw_size;
 wsi.handle=INVALID_HANDLE_VALUE;


 switch (wsi.portid)
 {
  case SERIAL_COM1:{prt[3]='1';break;}
  case SERIAL_COM2:{prt[3]='2';break;}
  default: return SERIAL_INVALIDPORT;
 }

 wsi.handle = CreateFile
                    (
                     prt,
                     GENERIC_READ | GENERIC_WRITE,
                     NULL,
                     NULL,
                     OPEN_EXISTING,
                     FILE_FLAG_OVERLAPPED,
                     NULL
                    );



 if (wsi.handle == INVALID_HANDLE_VALUE) {return SERIAL_INVALIDPORT;}
 if (!SetupComm(wsi.handle,wsi.rw_size,wsi.rw_size)) {return SERIAL_INVALIDPORT;}
 if (!GetCommState(wsi.handle, &wsi.dcb)) {return SERIAL_INVALIDPORT; }
 fillDcb(wsi,wsi.dcb);
 if (!SetCommState(wsi.handle, &wsi.dcb)){return SERIAL_INVALIDPORT; }
 CloseHandle(wsi.handle);

 wsi.handle = CreateFile
                    (
                     prt,
                     GENERIC_READ | GENERIC_WRITE,
                     NULL,
                     NULL,
                     OPEN_EXISTING,
                     FILE_FLAG_OVERLAPPED,
                     NULL
                    );



 if (wsi.handle == INVALID_HANDLE_VALUE) {return SERIAL_INVALIDPORT;}
 if (!SetupComm(wsi.handle,wsi.rw_size,wsi.rw_size)) {return SERIAL_INVALIDPORT;}
 if (!GetCommState(wsi.handle, &wsi.dcb)) {return SERIAL_INVALIDPORT; }
 fillDcb(wsi,wsi.dcb);
 if (!SetCommState(wsi.handle, &wsi.dcb)){return SERIAL_INVALIDPORT; }


 if (!GetCommTimeouts(wsi.handle,&wsi.CommTimeOuts)) {return SERIAL_INVALIDPORT; }

 wsi.CommTimeOuts.ReadIntervalTimeout         = 10;
 wsi.CommTimeOuts.ReadTotalTimeoutMultiplier  =  1;
 wsi.CommTimeOuts.ReadTotalTimeoutConstant    = 10;
 wsi.CommTimeOuts.WriteTotalTimeoutMultiplier =  0;
 wsi.CommTimeOuts.WriteTotalTimeoutConstant   =  0;

 if (!SetCommTimeouts(wsi.handle,&wsi.CommTimeOuts)) {return SERIAL_INVALIDPORT; }

 PurgeComm(wsi.handle,PURGE_RXCLEAR);
 PurgeComm(wsi.handle,PURGE_TXCLEAR);
 serial_ResetRTS(portid);
 serial_ResetDTR(portid);
 memmove(&::wsi[portid],&wsi,sizeof(wsi));
 return portid;
}

unsigned short serial_GetWBSize(unsigned short portid)
{
 ULONG temp;
 ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
 return (unsigned short)wsi[portid].ComState.cbOutQue;
}

unsigned short serial_GetRBSize(unsigned short portid)
{
 ULONG temp;
 ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
 return (unsigned short)wsi[portid].ComState.cbInQue;
}

unsigned short serial_GetWBFree(unsigned short portid)
{
 ULONG temp;
 ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
 return (unsigned short)wsi[portid].rw_size-wsi[portid].ComState.cbOutQue;
}
unsigned short serial_GetRBFree(unsigned short portid)
{
ULONG temp;
 ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
 return (unsigned short)wsi[portid].rw_size-wsi[portid].ComState.cbInQue;
}

unsigned short serial_Read(unsigned short portid,unsigned char *buffer,unsigned short size)
{
 ULONG temp;
 ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
 if (!temp)
 {
  BOOL retV=ReadFile(wsi[portid].handle,buffer,wsi[portid].ComState.cbInQue,&temp,&wsi[portid].Overlap);
  if (retV)
  {
   return (unsigned short)temp;
  }
 }
 return 0;
}

unsigned short serial_Write(unsigned short portid,unsigned char *buffer,unsigned short numbytes)
{
 DWORD temp;
 if (numbytes)
 {
   ClearCommError(wsi[portid].handle,&temp,&wsi[portid].ComState);
   if (numbytes+wsi[portid].ComState.cbOutQue>=wsi[portid].rw_size) MessageBox(NULL,"FFFF","",MB_OKCANCEL);
   BOOL retV=WriteFile(wsi[portid].handle,buffer,numbytes,&temp,&wsi[portid].Overlap);
   if (retV)
   {
      return (unsigned short)temp;
   }
 }
 return 0;
}

unsigned short ReadPortState(unsigned short portid,DWORD flag)
{
 DWORD ModemStat;
 GetCommModemStatus(wsi[portid].handle, &ModemStat);
 return (unsigned short)(ModemStat & flag);
}

unsigned char serial_ReadDSR(unsigned short portid)
{
 if (ReadPortState(portid,MS_DSR_ON)) return 1;
 else return 0;
}


unsigned char serial_ReadCTS(unsigned short portid)
{
 if (ReadPortState(portid,MS_CTS_ON)) return 1;
 else return 0;
}

unsigned char serial_ReadDCD(unsigned short portid)
{
 if (ReadPortState(portid,MS_RLSD_ON)) return 1;
 else return 0;
}

unsigned char serial_ReadRI(unsigned short portid)
{
 if (ReadPortState(portid,MS_RING_ON)) return 1;
 else return 0;
}

void serial_SetDTR(unsigned short portid)
{
 EscapeCommFunction(wsi[portid].handle,SETDTR);
}

void serial_ResetDTR(unsigned short portid)
{
 EscapeCommFunction(wsi[portid].handle,CLRDTR);
}

void serial_SetRTS(unsigned short portid)
{
 EscapeCommFunction(wsi[portid].handle,SETRTS);
}

void serial_ResetRTS(unsigned short portid)
{
 EscapeCommFunction(wsi[portid].handle,CLRRTS);
}

void serial_Reset(unsigned short portid)
{
 PurgeComm(wsi[portid].handle,PURGE_RXCLEAR);
 PurgeComm(wsi[portid].handle,PURGE_TXCLEAR);
 serial_ResetRTS(portid);
 serial_ResetDTR(portid);
}

//---------------------------------------------------------------------------------
void serial_ResetR(unsigned short portid)
{
 PurgeComm(wsi[portid].handle,PURGE_RXCLEAR);
}

void serial_ResetW(unsigned short portid)
{
 PurgeComm(wsi[portid].handle,PURGE_TXCLEAR);
}

unsigned short serial_Init(unsigned short portid,serial_TInit *cfg)
{
   return serial_Init(portid,cfg->speed,cfg->bit,cfg->stopbit,cfg->parity,cfg->rw_size);
}
